/**
 * JBoss, the OpenSource J2EE WebOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
 
package acme.client;

import javax.naming.InitialContext;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.Connection;
import javax.jms.Session;
import javax.jms.MessageProducer;
import javax.jms.Message;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;


public class JMSClient
{

   public static final int NUM_RETRIES = 3;
   volatile boolean doSend = true;
   ConnectionFactory connectionFactory;
   Destination topic;
   Connection connection;
   Session session;
   MessageProducer publisher;



   public static void main(String[] args) throws Exception
   {
      JMSClient jmsClient = new JMSClient();
      jmsClient.setUpJMS();
      jmsClient.sendMessages();
   }


   public boolean setUpJMS()
   {
      try
      {
         InitialContext ic = new InitialContext();

         connectionFactory = (ConnectionFactory)ic.lookup("ConnectionFactory");

         topic = (Destination)ic.lookup("topic/testDurableTopic");

         connection = connectionFactory.createConnection();

         connection.setExceptionListener(new ExceptionListenerImpl());

         session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

         publisher = session.createProducer(topic);

         return true;
      }
      catch(Throwable t)
      {
         System.out.println("Exception in setUpJMS()" +t);
        
         try { 
       	    connection.stop();
            session.close();
            connection.close();
            
         }catch(javax.jms.JMSException jmse){
         	   System.out.println(jmse);
         }
        
         return false;
      }
   }

   public void sendMessages()
   {
      int cnt = 0;
      while(doSend)
      {
         try
         {
            Thread.sleep(1000); //fake sleep, never do that in production code!!!!!!

            Message m = session.createObjectMessage(new Integer(cnt++));

            publisher.send(m);

            System.out.println("message " + cnt + " sent");
         }
         catch(Exception e)
         {
            cnt--;
            System.out.println(e.getMessage());
         }
      }
   }



   private class ExceptionListenerImpl implements ExceptionListener
   {
      public void onException(JMSException e)
      {

         for(int i = 0; i < NUM_RETRIES; i++)
         {
            System.out.println("Connection has problems, trying to re-create it, attempt " +
                               (i + 1) + " ...");

            boolean setupOK = setUpJMS();

            if (setupOK)
            {
               System.out.println("Connection re-established");
               return;
            }
            else
            {
               System.out.println("Re-creating connection failed, retrying ...");
            }
         }

         System.out.println("Cannot re-establish connection, giving up ...");
         doSend = false;
      }
   }
}
